"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.innerExtendsConfig = exports.normalizePlugins = void 0;
/**
 * @file extendsConfig
 * @author Cuttle Cong
 * @date 2018/3/24
 * @description
 */
const normalizeSource_1 = require("./normalizeSource");
const toArray_1 = require("../lib/toArray");
const extendsMerge_1 = require("./extendsMerge");
const loadConfig_1 = require("../lib/loadConfig");
const _ = require("lodash");
const preduce = require("p-reduce");
const nps = require("path");
const resolve_1 = require("../lib/resolve");
const fs = require("fs");
const untildify = require('untildify');
const debug = require('debug')('edam:extendsConfig');
function normalizePlugins(plugins, options) {
    plugins = toArray_1.default(plugins);
    return plugins.map(p => {
        function getPlugin(p) {
            const res = resolve_1.default(p, Object.assign(Object.assign({}, options), { safe: false, global: true }));
            debug('get Plugin: %s -> %s', p, res);
            return require(res);
        }
        if (_.isString(p)) {
            return [getPlugin(p), {}];
        }
        else if (_.isArray(p) && _.isString(p[0])) {
            return [getPlugin(p[0]), p[1]];
        }
        return p;
    });
}
exports.normalizePlugins = normalizePlugins;
function normalizeExtend(extend) {
    if (typeof extend === 'string') {
        return {
            source: extend,
            pick: [],
            omit: [],
        };
    }
    return Object.assign({ pick: [], omit: [] }, extend);
}
function innerExtendsConfig(config, options, track) {
    return __awaiter(this, void 0, void 0, function* () {
        let extendConfig;
        config = _.cloneDeep(config);
        debug('config %O', config);
        if (typeof config.output === 'string') {
            config.output = nps.resolve(options.cwd, config.output);
        }
        // source and alias could be relative path
        config.source = normalizeSource_1.default(config.source, options);
        if (config.alias) {
            _.each(config.alias, (val, key) => {
                config.alias[key] = normalizeSource_1.default(val, options);
            });
        }
        if (config.plugins) {
            config.plugins = normalizePlugins(config.plugins, options);
        }
        if (config.extends) {
            const extendsArray = (config.extends = toArray_1.default(config.extends));
            const configList = yield Promise.all(extendsArray.map((source) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const extendConfig = normalizeExtend(source);
                const sourcePath = nps.resolve(options.cwd || '', extendConfig.source);
                let result;
                if (fs.existsSync(sourcePath) && fs.statSync(sourcePath).isDirectory()) {
                    result = yield loadConfig_1.loadConfig(untildify(extendConfig.source), Object.assign(Object.assign({}, options), { filename: true, stopPath: sourcePath }));
                }
                else {
                    result = yield loadConfig_1.loadConfig(untildify(extendConfig.source), Object.assign(Object.assign({}, options), { filename: true }));
                }
                return Object.assign(Object.assign({}, result), { config: _.omit(!!((_a = extendConfig.pick) === null || _a === void 0 ? void 0 : _a.length) ? _.pick(result.config, extendConfig.pick) : result.config, extendConfig.omit) });
            })));
            extendConfig = yield preduce(configList, (collection, { config: innerConfig, invalid, filename }) => __awaiter(this, void 0, void 0, function* () {
                if (invalid) {
                    innerConfig = {};
                }
                let value;
                track[filename] = track[filename] || { status: 'first' };
                if (track[filename].status === 'visited') {
                    value = track[filename].value;
                }
                else if (track[filename].status === 'visiting') {
                    value = {};
                }
                else {
                    track[filename] = { status: 'visiting' };
                    value = yield innerExtendsConfig(innerConfig, Object.assign(Object.assign({}, options), { cwd: nps.dirname(filename) }), track);
                    track[filename] = {
                        status: 'visited',
                        value,
                    };
                }
                return extendsMerge_1.default(collection, value);
            }), {});
        }
        const result = extendsMerge_1.default({}, extendConfig, config);
        debug('extended track %O', track);
        debug('extended result %O', result);
        return result;
    });
}
exports.innerExtendsConfig = innerExtendsConfig;
function extendsConfig(config, options) {
    return __awaiter(this, void 0, void 0, function* () {
        options = Object.assign({ track: true }, options);
        const track = {};
        return {
            config: yield innerExtendsConfig(config, options, track),
            track: options.track ? track : null,
        };
    });
}
exports.default = extendsConfig;
