"use strict";
/* eslint-disable */
// Forked from https://github.com/jaz303/git-clone/blob/master/index.js
// use cross-spawn, instead spawn
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pullForce = exports.checkout = void 0;
const processAsync_2 = require("./processAsync");
const is_ci_1 = require("is-ci");
var pify = require("pify");
var spawn = require("cross-spawn");
var env = Object.assign(Object.assign({}, process.env), { GIT_TERMINAL_PROMPT: "0" });
function _checkout(checkout, { targetPath = "", git = "git" } = {}, cb) {
    var args = ["checkout", checkout];
    var process = spawn(git, args, { env, cwd: targetPath });
    processAsync_2.default(process, "git checkout", cb);
}
function clonePromise(repo, targetPath, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const cloneP = pify(_clone);
        try {
            return yield cloneP(repo, targetPath, opts);
        }
        catch (err) {
            if (opts.onDisablePromptError) {
                yield opts.onDisablePromptError(err);
            }
            if (!is_ci_1.default) {
                return yield cloneP(repo, targetPath, Object.assign(Object.assign({}, opts), { env: Object.assign(Object.assign({}, env), { GIT_TERMINAL_PROMPT: "1" }) }));
            }
        }
    });
}
function _clone(repo, targetPath, opts, cb) {
    if (typeof opts === "function") {
        cb = opts;
        opts = null;
    }
    opts = opts || {};
    var git = opts.git || "git";
    var args = ["clone"];
    if (opts.shallow) {
        args.push("--depth");
        args.push("1");
        args.push("--recurse-submodules");
        args.push("-j8");
    }
    args.push("--");
    args.push(repo);
    args.push(targetPath);
    var process = spawn(git, args, { env: opts.env || env });
    processAsync_2.default(process, "git clone", function (err, stdout) {
        if (err) {
            cb(err);
        }
        else {
            if (opts.checkout) {
                checkout();
            }
            else {
                cb && cb();
            }
        }
    });
    function checkout() {
        _checkout(opts.checkout, { targetPath, git }, cb);
    }
}
function _pullForce(targetPath, outerEnv = env, cb) {
    var process = spawn("git", ["pull", "--force", "--allow-unrelated-histories"], {
        cwd: targetPath,
        env: outerEnv
    });
    processAsync_2.default(process, "git pull", cb);
}
function _reset(targetPath, outerEnv = env, cb) {
    var process = spawn("git", ["reset", "--hard", "HEAD"], {
        cwd: targetPath,
        env: outerEnv
    });
    processAsync_2.default(process, "git reset", cb);
}
function _cleanFD(targetPath, outerEnv = env, cb) {
    var process = spawn("git", ["clean", "-f", "-d"], {
        cwd: targetPath,
        env: outerEnv
    });
    processAsync_2.default(process, "git clean", cb);
}
const _pullForcePromise = pify(_pullForce);
const reset = pify(_reset);
const cleanFD = pify(_cleanFD);
exports.default = clonePromise;
exports.checkout = pify(_checkout);
const pullForce = (targetPath, opts) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        yield reset(targetPath, env);
        yield cleanFD(targetPath, env);
        yield _pullForcePromise(targetPath, env);
    }
    catch (err) {
        if (opts && opts.onDisablePromptError) {
            yield opts.onDisablePromptError(err);
        }
        if (!is_ci_1.default) {
            yield reset(targetPath, env);
            yield cleanFD(targetPath, env);
            return yield _pullForcePromise(targetPath, Object.assign(Object.assign({}, env), { GIT_TERMINAL_PROMPT: "1" }));
        }
    }
});
exports.pullForce = pullForce;
