"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = exports.load = void 0;
const safeRequireResolve_1 = require("./safeRequireResolve");
const EdamError_1 = require("../core/EdamError");
const JSON5 = require('json5');
const cosmiconfig = require('cosmiconfig');
const nps = require('path');
const fs = require('fs');
const fileSystem = require('./fileSystem').default;
const tildify = require('tildify');
const explorer = cosmiconfig('edam', { rcStrictJson: true });
const debug = require('debug')('edam:loadConfig');
function parseJSONFile(filename) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const text = yield fileSystem.readFile(filename, { encoding: 'utf8' });
            return JSON5.parse(text);
        }
        catch (err) {
            throw new EdamError_1.default(`${err.message} in file: "${tildify(filename)}"`);
        }
    });
}
function getMatchJSONErrorFilename(err) {
    // Error: Failed to parse "/Users/yucong02/self/edam/packages/edam/src/__test__/fixture/loadConfig/.filerc" as JSON, JS, or YAML.
    if (/Failed to parse "(.+)" as JSON, JS, or YAML\.$/.test(err.message)) {
        return RegExp.$1;
    }
    if (/JSON Error in (.+):/.test(err.message)) {
        return RegExp.$1;
    }
}
function load(searchPath) {
    return __awaiter(this, void 0, void 0, function* () {
        debug('load searchPath: %s', searchPath);
        try {
            const obj = yield explorer.load(searchPath);
            debug('load : %o', obj);
            return obj;
        }
        catch (err) {
            const filename = getMatchJSONErrorFilename(err);
            if (filename) {
                return {
                    config: yield parseJSONFile(filename),
                    filepath: filename,
                };
            }
            throw err;
        }
    });
}
exports.load = load;
function loadConfig(path, options = {
    cwd: process.cwd(),
    filename: false,
}) {
    return __awaiter(this, void 0, void 0, function* () {
        const { cwd, stopPath } = options;
        const isValidConfigFilename = filename => {
            if (!stopPath) {
                return !!filename;
            }
            return nps.dirname(filename).startsWith(nps.normalize(stopPath));
        };
        let isDir = false;
        // require npm packages directly
        let resolved = safeRequireResolve_1.default(path);
        let filename;
        if (resolved) {
            filename = resolved;
        }
        else {
            filename = nps.resolve(cwd, path);
            if (fs.existsSync(filename) && fs.statSync(filename).isDirectory()) {
                isDir = true;
            }
            else {
                filename = require.resolve(filename);
            }
        }
        let resultConfig;
        try {
            const { config, filepath } = yield (isDir ? explorer.load(filename) : explorer.load(null, filename));
            resultConfig = config;
            filename = filepath;
        }
        catch (err) {
            filename = getMatchJSONErrorFilename(err);
            if (filename) {
                resultConfig = yield parseJSONFile(filename);
            }
            else {
                throw err;
            }
        }
        if (!isValidConfigFilename(filename)) {
            return options.filename ? { config: null, filename, invalid: true } : null;
        }
        return options.filename ? { config: resultConfig, filename } : resultConfig;
    });
}
exports.loadConfig = loadConfig;
