import { Edam } from '../index';
export declare type AsyncOrSync<T> = Promise<T> | T;
export declare type PromptType = 'checkbox' | 'radio' | 'input' | 'suggest' | any;
export interface Prompt {
    message: string;
    default?: any;
    yes?: boolean;
    when?: (vars: Variables) => boolean;
    name: string;
    type: PromptType;
    choices?: Array<any>;
    transform?: Function;
    deniesStore?: boolean;
    transformer?: (val: any, set: Record<string, any>, context: Record<string, any>) => any;
    validate?: (val: any, set: Record<string, any>, context: Record<string, any>) => string | boolean;
}
export declare type Hook = string | ((...args: any[]) => void);
export declare type Glob = string;
export declare type VarCenter = {
    once: Function;
    always: Function;
};
export declare type GetVariable = (vc: VarCenter) => AsyncOrSync<any>;
export declare type Variable = GetVariable | any;
export declare type Variables = {
    [name: string]: Variable;
};
export declare type StrictLoader = Function & {
    raw?: boolean;
};
export declare type StrictLoaderWithOption = [StrictLoader, any];
export declare type Loader = Array<StrictLoader | string | StrictLoaderWithOption> | string | StrictLoader | StrictLoaderWithOption;
export declare type Matcher = Glob | Glob[] | RegExp | Function;
export declare type Mapper = {
    test?: Matcher;
    mimeTest?: Matcher;
    loader: Loader;
};
export declare type Dynamic<T> = (answer: object) => T | Promise<T>;
export declare function defineConfig(config: UserTemplateConfig): UserTemplateConfig;
export declare type UserTemplateConfig = TemplateConfig | ((edam: Edam) => TemplateConfig);
export default interface TemplateConfig extends TemplateGeneratingConfig {
    prompts?: Array<Prompt>;
    /**
     * answers => ({ hooks, ignore, ...(exclude prompts) })
     */
    process?: (answers: any) => TemplateGeneratingConfig;
}
export interface TemplateGeneratingConfig {
    hooks?: {
        [hookName: string]: Array<Hook> | Hook;
    } | Dynamic<{
        [hookName: string]: Array<Hook> | Hook;
    }>;
    ignore?: string[] | Dynamic<string[]>;
    variables?: Variables | Dynamic<Variables>;
    root?: string | Dynamic<string>;
    loaders?: {
        [loaderId: string]: Array<StrictLoader>;
    };
    mappers?: Array<Mapper>;
    move?: Dynamic<Record<string, string>> | Record<string, string>;
    copy?: Dynamic<Record<string, string>> | Record<string, string>;
    usefulHook?: {
        gitInit?: boolean;
        installDependencies?: boolean;
        installDevDependencies?: boolean;
    };
}
